/* @source domainer application
**
** Reads protein coordinate files and writes domains coordinate files.
**
** @author: Copyright (C) Jon Ison (jison@hgmp.mrc.ac.uk)
** @@
**
** This program is free software; you can redistribute it and/or
** modify it under the terms of the GNU General Public License
** as published by the Free Software Foundation; either version 2
** of the License, or (at your option) any later version.
**
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program; if not, write to the Free Software
** Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*******************************************************************************
**
**
**
*******************************************************************************
**IMPORTANT NOTE      IMPORTANT NOTE      IMPORTANT NOTE        IMPORTANT NOTE
*******************************************************************************
**
** Mon May 20 11:43:39 BST 2002
**
** The following documentation is out-of-date and should be disregarded.  It
** will be updated shortly.
**
*******************************************************************************
**IMPORTANT NOTE      IMPORTANT NOTE      IMPORTANT NOTE        IMPORTANT NOTE
*******************************************************************************
**
**
**
** Operation
**
** domainer parses an embl-like format scop classification file generated by
** the EMBOSS applications scope or nrscope, and clean protein coordinate files
** generated by the coorde application (not currently in emboss, email Jon Ison
** jison@hgmp.mrc.ac.uk) and writes, for each domain in the scop classification
** file, clean domain coordinate files in embl-like and pdb formats .  Each of
** these files contains coordinates for a single scop domain.  In cases
** where multiple models were determined, the data in the domain files
** correspond to the first model.  In the rare cases where a domain is
** comprised of more than one chain, the data will be presented as belonging
** to a single chain (i.e. a single sequence, chain identifier etc will be
** given).
**
**
** The pdb format is explained at URL (1)
** (1) http://www.rcsb.org/pdb/docs/format/pdbguide2.2/guide2.2_frame.html
** domainer writes the following records in pdb format (Figure 1):
** (1)  HEADER - bibliographic information.  The text 'CLEANED-UP PDB FILE FOR
** SCOP DOMAIN XXXXXXX' is always given (where XXXXXXX is a 7-character domain
** identifier code).
** (2)  TITLE - bibliographic information. The text ' THIS FILE IS MISSING
** MOST RECORDS FROM THE ORIGINAL PDB FILE' is always given.
** (3)  COMPND - compound information.  The COMPND records from the original
** pdb file are given.
** (4)  SOURCE - protein source information.  The SOURCE records from the
** original pdb file are given.
** (5)  REMARK - remark records. Remark records are used for spacing. One
** REMARK line containing the protein resolution is always given.
** (6)  SEQRES - protein sequence.
** (7)  ATOM - atomic coordinates.
** (8)  TER - indicates the end of a chain.
**
**
**
** Figure 1  Excerpt from clean coordinate file (pdb format)
** HEADER     CLEANED-UP PDB FILE FOR SCOP DOMAIN D1HBBA_
**  TITLE      THIS FILE IS MISSING MOST RECORDS FROM THE ORIGINAL PDB FILE
**  COMPND     HEMOGLOBIN A (DEOXY, LOW SALT, 100MM CL)
**  SOURCE     HUMAN (HOMO SAPIENS)
**  REMARK
**  REMARK     RESOLUTION. 1.90  ANGSTROMS.
**  REMARK
**  SEQRES   1 A  141  VAL LEU SER PRO ALA ASP LYS THR ASN VAL LYS ALA ALA
**  SEQRES   2 A  141  TRP GLY LYS VAL GLY ALA HIS ALA GLY GLU TYR GLY ALA
**  SEQRES   3 A  141  GLU ALA LEU GLU ARG MET PHE LEU SER PHE PRO THR THR
**  SEQRES   4 A  141  LYS THR TYR PHE PRO HIS PHE ASP LEU SER HIS GLY SER
**  SEQRES   5 A  141  ALA GLN VAL LYS GLY HIS GLY LYS LYS VAL ALA ASP ALA
**  SEQRES   6 A  141  LEU THR ASN ALA VAL ALA HIS VAL ASP ASP MET PRO ASN
**  SEQRES   7 A  141  ALA LEU SER ALA LEU SER ASP LEU HIS ALA HIS LYS LEU
**  SEQRES   8 A  141  ARG VAL ASP PRO VAL ASN PHE LYS LEU LEU SER HIS CYS
**  SEQRES   9 A  141  LEU LEU VAL THR LEU ALA ALA HIS LEU PRO ALA GLU PHE
**  SEQRES  10 A  141  THR PRO ALA VAL HIS ALA SER LEU ASP LYS PHE LEU ALA
**  SEQRES  11 A  141  SER VAL SER THR VAL LEU THR SER LYS TYR ARG
**  ATOM      1  N   VAL A   1       7.155  17.725   4.424  1.00 37.82        N
**  ATOM      2  CA  VAL A   1       7.854  18.800   3.718  1.00 35.10        C
**  ATOM      3  C   VAL A   1       9.366  18.565   3.754  1.00 31.92        C
**  ATOM      4  O   VAL A   1       9.861  17.961   4.721  1.00 35.01        O
**  ATOM      5  CB  VAL A   1       7.529  20.168   4.360  1.00 47.63        C
**  ATOM      6  CG1 VAL A   1       7.806  21.300   3.369  1.00 62.84        C
**  ATOM      7  CG2 VAL A   1       6.136  20.244   4.936  1.00 54.85        C
**  ATOM      8  N   LEU A   2      10.032  19.062   2.731  1.00 27.38        N
**  ATOM      9  CA  LEU A   2      11.496  18.967   2.657  1.00 23.24        C
**  ATOM     10  C   LEU A   2      12.077  20.110   3.496  1.00 22.99        C
**  ATOM     11  O   LEU A   2      11.672  21.259   3.289  1.00 25.22        O
**  ATOM     12  CB  LEU A   2      11.924  19.005   1.204  1.00 18.04        C
**  ATOM     13  CG  LEU A   2      11.563  17.855   0.286  1.00 17.80        C
**  ATOM     14  CD1 LEU A   2      12.166  18.109  -1.097  1.00 20.08        C
**  ATOM     15  CD2 LEU A   2      12.116  16.542   0.839  1.00 13.84        C
**  ATOM     16  N   SER A   3      12.979  19.784   4.391  1.00 22.22        N
**  ATOM     17  CA  SER A   3      13.652  20.792   5.257  1.00 20.53        C
**  ATOM     18  C   SER A   3      14.871  21.318   4.505  1.00 18.31        C
**  ATOM     19  O   SER A   3      15.273  20.709   3.496  1.00 17.73        O
**  ATOM     20  CB  SER A   3      14.084  20.042   6.534  1.00 17.61        C
**
**
** The embl-like format used for clean domain coordinate files (Figure
** 2) uses the following records:
**
** (1)  ID - Either the 4-character PDB identifier code (for clean protein
** coordinate files) or the 7-character domain identifier code taken from scop
** (for domain coordinate files; see documentation for the EMBOSS application
** scope for further info.)
**
** (2)  DE - compound information.  Text from the COMPND records from the
** original pdb file are given.
**
** (3)  OS - protein source information.  Text from the SOURCE records from the
** original pdb file are given.
**
** (4)  EX - experimental information. The text 'nmr_or_model' (for nuclear
** magnetic resonance and model structures) or 'xray' (for structures
** determined by X-ray crystallography) appears as appropriate after the text
** 'METHOD'.  The resolution of X-ray structures, or '0' for structures of type
** 'nmr_or_model', is given after 'RESO'.  The number of models and number of
** polypeptide chains are given after 'NMOD' and 'NCHA' respectively. For
** domain coordinate files a 1 is always given. Following the EX record, the
** file will have a section containing a CN, IN and SQ records (see below) for
** each chain.
**
** (5)  CN - chain number.  The number given in brackets after this record
** indicates the start of a section of chain-specific data.
**
** (6)  IN - chain specific data. The character given after ID is the PDB chain
** identifier (a '.' is given in cases where a chain identifier was not
** specified in the pdb file or, for domain coordinate files, the domain is
** comprised of more than one domain).  The number of amino acid residues
** comprising the chain (or the chains from which a domain is comprised) is
** given after NR. The number of atoms in heterogens and water molecules are
** given after NH and NW respectively. Domain coordinate files do not include
** coordinates for these groups so a value of 0 is always given.
**
** (7)  SQ - protein sequence. The number of residues is given before AA on the
** first line. The protein sequence is given on subsequent lines.
**
** (8)  CO - coordinate data.  The records are as follows (column numbers are
** given in parentheses). (1) CO is always given. (2) Model number (always 1
** for domain coordinate files). (3) Chain number (always 1 for domain
** coordinate files). (4) Either P (a protein atom), H (a heterogen atom) or W
** (an atom in a water molecule).  (5) Position of the residue in the protein
** sequence given in the SQ record (for protein atoms) or a sequential count
** of the atoms (for heterogens and water). (6) Residue number according to the
** original pdb file, or or a sequential count of the atoms (for heterogens and
** water). (7) Single character amino acid code or a '.' (for heterogens and
** water).  (8) 3-character residue identifier code.  (9) Atom type. (10-12)
** The x, y and z orthogonal coordinates. (13) Occupancy. (14) Temperature
** factor.
**
** (9)  XX - Used for spacing.
**
** (10) // - Given on the last line of the file only.
**
**
** Figure 2 Excerpt from clean domain coordinate file (embl-like format)
**  ** ID   D1HBBA_
**  XX
**  DE   Co-ordinates for SCOP domain D1HBBA_
**  XX
**  OS   See Escop.dat for domain classification
**  XX
**  EX   METHOD xray; RESO 1.90; NMOD 1; NCHA 1;
**  XX
**  CN   [1]
**  XX
**  IN   ID A; NR 141; NH 0; NW 0;
**  XX
**  SQ   SEQUENCE   141 AA;  15127 MW;  5EC7DB1E CRC32;
**       VLSPADKTNV KAAWGKVGAH AGEYGAEALE RMFLSFPTTK TYFPHFDLSH GSAQVKGHGK
**       KVADALTNAV AHVDDMPNAL SALSDLHAHK LRVDPVNFKL LSHCLLVTLA AHLPAEFTPA
**       VHASLDKFLA SVSTVLTSKY R
**  XX
**  CO   1  1  P  1   1   V  VAL  N    7.155 17.725  4.424   1.00  37.82
**  CO   1  1  P  1   1   V  VAL  CA   7.854 18.800  3.718   1.00  35.10
**  CO   1  1  P  1   1   V  VAL  C    9.366 18.565  3.754   1.00  31.92
**  CO   1  1  P  1   1   V  VAL  O    9.861 17.961  4.721   1.00  35.01
**  CO   1  1  P  1   1   V  VAL  CB   7.529 20.168  4.360   1.00  47.63
**  CO   1  1  P  1   1   V  VAL  CG1  7.806 21.300  3.369   1.00  62.84
**  CO   1  1  P  1   1   V  VAL  CG2  6.136 20.244  4.936   1.00  54.85
**  CO   1  1  P  2   2   L  LEU  N   10.032 19.062  2.731   1.00  27.38
**  CO   1  1  P  2   2   L  LEU  CA  11.496 18.967  2.657   1.00  23.24
**  CO   1  1  P  2   2   L  LEU  C   12.077 20.110  3.496   1.00  22.99
**  CO   1  1  P  2   2   L  LEU  O   11.672 21.259  3.289   1.00  25.22
**
**
** domainer generates a log file an excerpt of which is shown (Figure 3). If
** there is a problem in processing a domain, three lines containing the
** record '//', the domain identifier code and an error message respectively
** are written. The text 'WARN  filename not found' is given in cases where a
** clean coordinate file could not be found. 'ERROR filename file read error'
** or 'ERROR filename file write error' will be reported when an error was
** encountered during a file read or write respectively.  The 'ERROR Domain
** start found by wildcard match only' is reported when wildcard matching
** was needed to find the start (or end) of a domain in a pdb file (see below).
** Various other error messages may also be given (in case of difficulty
** email Jon Ison, jison@hgmp.mrc.ac.uk).
**
** Figure 3 Excerpt of log file
** //
** DS002__
** WARN  Could not open for reading cpdb file s002.pxyz
** //
** DS003__
** WARN  Could not open for reading cpdb file s003.pxyz
**
** Important Note
** Messages of the type:
** //
** D0LPC_1
** WARN  0lpc.pxyz not found
**
** can appear in the log file if
** (i)  The domain is of a pdb file which is in holding, but has not made it
** into the main pdb release yet.
** (ii) The domain is of a pdb file that is now obsolete - having been replaced
** by a more recent entry.
** There may be other cases too.
**
**
** Messages of the type:
** The start and end positions of domains in SCOP coincide in several cases
** to residues which either lack a CA atom or which have a single atom only
** structured. Therefore to avoid these domains being rejected during a
** domainer run, it is important to use clean coordinate data which has NOT
** been masked to remove such residues.
** Otherwise errors of the following type may occur:
** //
** D1QFUL1
** ERROR Domain end not found in ajXyzCpdbWriteDomain
**
** //
** D1QFUL1
** ERROR Domain start not found in ajXyzCpdbWriteDomain
**
**
** Wildcard matching of scop domain start and end points
** SCOP treats pdb residue numbers as integers and this is a potential source
** of error in the domain ranges. It means that domainer (via library calls)
** must use wild card strings when matching a scop start / end point to the
** pdb residue number strings.
******************************************************************************/






#include "emboss.h"







/* @prog domainer *************************************************************
**
** Build domain coordinate files
**
******************************************************************************/

int main(int argc, char **argv)
{
    AjPStr   cpdb_path     =NULL;	/* Path of cpdb files */
    AjPStr   cpdb_extn     =NULL;	/* Extn. of cpdb files */
    AjPStr   pdb_extn      =NULL;	/* Extn. of pdb files */
    AjPStr   pdbscop_path  =NULL;	/* Path of pdbscop files */
    AjPStr   cpdbscop_path =NULL;	/* Path of cpdbscop files */
    AjPStr   cpdb_name     =NULL;	/* Name of cpdb file */
    AjPStr   pdbscop_name  =NULL;	/* Name of pdbscop file */
    AjPStr   cpdbscop_name =NULL;	/* Name of cpdbscop file */
    AjPStr   msg           =NULL;	/* Error message */
    AjPStr   temp          =NULL;	/* Error message */

    AjPFile  scop_inf      =NULL;
    AjPFile  cpdb_inf      =NULL;
    AjPFile  pdbscop_outf  =NULL;
    AjPFile  cpdbscop_outf =NULL;
    AjPFile  errf1         =NULL;
    AjPFile  errf2         =NULL;

    AjPScop  scop=NULL;
    AjPPdb   pdb=NULL;


    /* Initialise strings */
    msg           = ajStrNew();
    cpdb_path     = ajStrNew();
    cpdb_extn     = ajStrNew();
    pdb_extn      = ajStrNew();
    pdbscop_path  = ajStrNew();
    cpdbscop_path = ajStrNew();
    cpdb_name     = ajStrNew();
    pdbscop_name  = ajStrNew();
    cpdbscop_name = ajStrNew();
    temp          = ajStrNew();


    /* Read data from acd */
    embInit("domainer",argc,argv);
    cpdb_path     = ajAcdGetString("cpdb");
    cpdb_extn     = ajAcdGetString("cpdbextn");
    pdb_extn      = ajAcdGetString("pdbextn");
    pdbscop_path  = ajAcdGetString("pdbscop");
    cpdbscop_path = ajAcdGetString("cpdbscop");
    scop_inf      = ajAcdGetInfile("scop");
    errf1         = ajAcdGetOutfile("pdberrf");
    errf2         = ajAcdGetOutfile("cpdberrf");


    /* Check directories*/
    if(!ajFileDir(&cpdb_path))
	ajFatal("Could not open cpdb directory");

    if(!ajFileDir(&pdbscop_path))
	ajFatal("Could not open pdbscop directory");

    if(!ajFileDir(&cpdbscop_path))
	ajFatal("Could not open cpdbscop directory");






    /*Start of main application loop*/
    while((ajXyzScopReadC(scop_inf, "*", &scop)))
    {
	/* Write diagnostic */
	ajFmtPrint("%S\n", scop->Entry);


	/* Read clean coordinate file*/
	ajStrAss(&cpdb_name, scop->Pdb);
	ajStrToLower(&cpdb_name);
	ajStrApp(&cpdb_name, cpdb_extn);
	if(!(cpdb_inf=ajFileNewDF(cpdb_path, cpdb_name)))
	{
	    ajFmtPrintS(&msg, "Could not open for reading cpdb file %S",
			cpdb_name);
	    ajWarn(ajStrStr(msg));
	    ajFmtPrintF(errf1, "//\n%S\nWARN  %S not found\n",
			scop->Entry, cpdb_name);
	    ajFmtPrintF(errf2, "//\n%S\nWARN  %S not found\n",
			scop->Entry, cpdb_name);
	    ajFileClose(&cpdb_inf);
	    ajXyzScopDel(&scop);
	    continue;
	}


	/* Write pdb structure */
	if(!ajXyzCpdbRead(cpdb_inf, &pdb))
	{
	    ajFmtPrintS(&msg, "Error reading cpdb file %S", cpdb_name);
	    ajWarn(ajStrStr(msg));
	    ajFmtPrintF(errf1, "//\n%S\nERROR %S file read error\n",
			scop->Entry, cpdb_name);
	    ajFmtPrintF(errf2, "//\n%S\nERROR %S file read error\n",
			scop->Entry, cpdb_name);
	    ajFileClose(&cpdb_inf);
	    ajXyzScopDel(&scop);
	    ajXyzPdbDel(&pdb);
	    continue;
	}


	/* Open pdb format file for writing*/
	ajStrAss(&pdbscop_name, pdbscop_path);
	ajStrApp(&pdbscop_name, scop->Entry);
	ajStrToLower(&pdbscop_name);
	ajStrAppC(&pdbscop_name, ajStrStr(pdb_extn));
	if(!(pdbscop_outf=ajFileNewOut(pdbscop_name)))
	{
	    ajFmtPrintS(&msg, "Could not open for writing pdbscop file %S",
			pdbscop_name);
	    ajWarn(ajStrStr(msg));
	    ajFmtPrintF(errf1, "//\n%S\nERROR %S file write error\n",
			scop->Entry, pdbscop_name);
	    ajFileClose(&cpdb_inf);
	    ajFileClose(&pdbscop_outf);
	    ajXyzScopDel(&scop);
	    ajXyzPdbDel(&pdb);
	    continue;
	}


	/* Open embl-like format file for writing*/
	ajStrAss(&cpdbscop_name, cpdbscop_path);
	ajStrApp(&cpdbscop_name, scop->Entry);
	ajStrToLower(&cpdbscop_name);
	ajStrApp(&cpdbscop_name, cpdb_extn);
	if(!(cpdbscop_outf=ajFileNewOut(cpdbscop_name)))
	{
	    ajFmtPrintS(&msg, "Could not open for writing cpdbscop file %S",
			cpdbscop_name);
	    ajWarn(ajStrStr(msg));
	    ajFmtPrintF(errf2, "//\n%S\nERROR %S file write error\n",
			scop->Entry, cpdbscop_name);
	    ajFileClose(&cpdb_inf);
	    ajFileClose(&cpdbscop_outf);
	    ajXyzScopDel(&scop);
	    ajXyzPdbDel(&pdb);
	    continue;
	}


	/* Write domain coordinate file in pdb format */
	if(!ajXyzPdbWriteDomain(errf1, pdbscop_outf, pdb, scop))
	{
	    ajFmtPrintS(&msg, "Error writing pdbscop file %S", pdbscop_name);
	    ajWarn(ajStrStr(msg));

	    ajFileClose(&pdbscop_outf);
	    ajFmtPrintS(&temp, "rm %S", pdbscop_name);
	    ajFmtPrint("%S", temp);
	    ajSystem(&temp);

	}


	/* Write domain coordinate file in embl-like format */
	if(!ajXyzCpdbWriteDomain(errf2, cpdbscop_outf, pdb, scop))
	{
	    ajFmtPrintS(&msg, "Error writing cpdbscop file %S",
			cpdbscop_name);
	    ajWarn(ajStrStr(msg));

	    ajFileClose(&cpdbscop_outf);
	    ajFmtPrintS(&temp, "rm %S", cpdbscop_name);
	    ajFmtPrint("%S", temp);

	    ajSystem(&temp);
	}


	/* Tidy up*/
	ajFileClose(&cpdb_inf);
	ajFileClose(&pdbscop_outf);
	ajFileClose(&cpdbscop_outf);
	ajXyzScopDel(&scop);
	ajXyzPdbDel(&pdb);
    }
    /*End of main application loop*/





    /*Tidy up */
    ajStrDel(&cpdb_path);
    ajStrDel(&cpdb_extn);
    ajStrDel(&pdb_extn);
    ajStrDel(&pdbscop_path);
    ajStrDel(&cpdbscop_path);
    ajStrDel(&cpdb_name);
    ajStrDel(&pdbscop_name);
    ajStrDel(&cpdbscop_name);
    ajStrDel(&msg);
    ajStrDel(&temp);
    ajFileClose(&scop_inf);
    ajFileClose(&errf1);
    ajFileClose(&errf2);


    /* Bye Bye */
    ajExit();
    return 0;
}
